/******************************************************************************/
/*                                                                            */
/* Licensed Materials - Property of IBM                                       */
/* (C) Copyright IBM Corp. 2000                                               */
/*                                                                            */
/* Progarm name    : IMSMSGHandler.h                                          */
/*                                                                            */
/* Description     : This header file contains function prototypes and        */
/*                   constants for IMSMSGHandler.c                            */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Version 1.0                                                                */
/*                                                                            */
/******************************************************************************/
  FILE *fptr;

/* constants */

#define MAX_ATTRIBUTE_NAME_LEN     128
#define MAX_NODE_NAME_LEN          128
#define MAX_TERMINAL_NAME_LEN      256
#define MAX_STRING_SIZE            256

#define CNI_TYPE_INTEGER 1
#define CNI_TYPE_SHORT   2
#define CNI_TYPE_CHAR    3

#define MSG_IMS_IIH     1
#define MSG_NO_IMS      2
#define MSG_IMS_NO_IIH  3

#define TOIMS     1
#define FROMIMS   2

#define OUTPUT_TERM  1
#define FAILURE_TERM 2

#define OUTMSG 1
#define INMSG  2

#define BIP2623 ((DWORD)0x40000A3FL)

#define VALUEBYTEARRAYSIZE 1024

#define CCI_CHECK_RC()                                                         \
   if (rc != CCI_SUCCESS)                                                      \
   {                                                                           \
     if (rc == CCI_EXCEPTION) cniRethrowLastException(0);                      \
     else                                                                      \
	 {                                                                         \
       cniLog(                                                                 \
              &rc                                                              \
             ,CCI_LOG_ERROR                                                    \
			 ,__FILE__                                                         \
			 ,__LINE__                                                         \
			 ,""                                                               \
			 ,0                                                                \
			 ,"Message not propagated"                                         \
			 ,0                                                                \
			 );                                                                \
       return;                                                                 \
     }                                                                         \
   }

/* typedefs */

typedef struct attributeTableEntry
{
  void    *next;                         /* Pointer to next entry */
  void    *valuePtr;                     /* Pointer to value */
  int      type;                         /* Value type */
  wchar_t  name[MAX_ATTRIBUTE_NAME_LEN]; /* Attribute name */
} ATTRIBUTE_TABLE_ENTRY;

typedef struct terminalListEntry
{
  void    *next;                        /* Pointer to next entry */
  void    *handle;                      /* Handle to terminal object */
  wchar_t  name[MAX_TERMINAL_NAME_LEN]; /* Terminal name */
} TERMINAL_LIST_ENTRY;

typedef struct context
{
  void *nodeObject;                                 /* Handle for node object */
  ATTRIBUTE_TABLE_ENTRY* attributeListHead;         /* Head of linked list of attributes */
  ATTRIBUTE_TABLE_ENTRY* attributeListPrevious;     /* Previous entry in attribute list */
  TERMINAL_LIST_ENTRY* inputTerminalListHead;       /* Head of linked list of input terminal handles */
  TERMINAL_LIST_ENTRY* inputTerminalListPrevious;   /* Previous entry in input terminal list */
  TERMINAL_LIST_ENTRY* outputTerminalListHead;      /* Head of linked list of output terminal handles */
  TERMINAL_LIST_ENTRY* outputTerminalListPrevious;  /* Previous entry in output terminal list */
  wchar_t nodeName[MAX_NODE_NAME_LEN];              /* Node name */
} CONTEXT_STRUCTURE;

#if defined(__CCI_SOLARIS__) 
/* ---------------------------------------------------------------- */
/*                     Internal constants                           */
/* ---------------------------------------------------------------- */
#define CONST_IMS_NODE                    "IMSMSGHandlerNode"
const CciChar * constIMSNode                = 0;

#define CONST_ZERO                           "0"
const CciChar * constZero                   = 0;

#define CONST_TYPE                           "type"
const CciChar * constType                   = 0;

#define CONST_MODIFY                         "modify"
const CciChar * constModify                 = 0;

#define CONST_OUT                            "out"
const CciChar * constOut                    = 0;

#define CONST_ADD                            "add"
const CciChar * constAdd                    = 0;

#define CONST_CHANGE                         "change"
const CciChar * constChange                 = 0;

#define CONST_DELETE                         "delete"
const CciChar * constDelete                 = 0;

#define CONST_HOLD                           "hold"
const CciChar * constHold                   = 0;

#define CONST_FAILURE                        "failure"
const CciChar * constFailure                = 0;
#endif


/* Function Prototypes */

/* Node Utility Functions */
ATTRIBUTE_TABLE_ENTRY *insertAttributeTableEntry
(
  CONTEXT_STRUCTURE *context,
  wchar_t    *nodeName, 
  wchar_t    *attributeName,
  int         type
);

void *getInputTerminalHandle
(
  CONTEXT_STRUCTURE *context,
  wchar_t    *terminalName
);

TERMINAL_LIST_ENTRY *insertInputTerminalListEntry
(
  CONTEXT_STRUCTURE *context,
  wchar_t *terminalName 
);

void *getOutputTerminalHandle
(
  CONTEXT_STRUCTURE *context,
  wchar_t *terminalName
);

TERMINAL_LIST_ENTRY *insertOutputTerminalListEntry
(
  CONTEXT_STRUCTURE *context,
  wchar_t *terminalName 
);

void traverseMessage
(
  void *element,
  FILE* outputFile
);

void test(void *element);

/* Node Implementation Functions */
void *createNodeContext(
                        void    *factoryObject,
                        wchar_t *nodeName,
                        void    *nodeObject
                       );

int getAttributeName(
                     void*    context,
                     int      index,
                     wchar_t* buffer,
                     int      bufferSize
                    );

int setAttribute(
                 void    *context,
                 wchar_t *attributeName,
                 wchar_t *attributeValue
                );

int getAttribute(
                 void    *context,
                 wchar_t *attributeName,
                 wchar_t *buffer,
                 int      bufferSize
                );

void evaluate(
              void *context, 
              void *destinationList,
              void *exceptionList,
              void *message
             );